@@ -4,7 +4,7 @@
 #include "osabi.h"
 #include "target-descriptions.h"
 
-const struct target_desc *tdesc_or1k;
+const_target_desc_up tdesc_or1k;
 static void
 initialize_tdesc_or1k (void)
 {
@@ -71,5 +71,5 @@ initialize_tdesc_or1k (void)
   tdesc_create_reg (feature, "npc", 33, 1, NULL, 32, "code_ptr");
   tdesc_create_reg (feature, "sr", 34, 1, NULL, 32, "sr_flags");
 
-  tdesc_or1k = result.release ();
+  tdesc_or1k = std::move (result);
 }
